/*
ADXL345 lib sample

copyright (c) Davide Gironi, 2012

Released under GPLv3.
Please refer to LICENSE file for licensing information.
*/


#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <math.h>

#include "adxl345/adxl345.h"

#define UART_BAUD_RATE 2400
#include "uart/uart.h"

#define DEBUG_ENABLED 0 //enable or disable debug output

#define CRCCHECK_ENABLED 1 //enable or disable crc check

#if CRCCHECK_ENABLED == 1
//crc8 lookup for poly X^8 + X^7 + X^2 + 1
static const unsigned char PROGMEM crc8table[256] =
{
  0x00,0x85,0x8F,0x0A,0x9B,0x1E,0x14,0x91,0xB3,0x36,
  0x3C,0xB9,0x28,0xAD,0xA7,0x22,0xE3,0x66,0x6C,0xE9,
  0x78,0xFD,0xF7,0x72,0x50,0xD5,0xDF,0x5A,0xCB,0x4E,
  0x44,0xC1,0x43,0xC6,0xCC,0x49,0xD8,0x5D,0x57,0xD2,
  0xF0,0x75,0x7F,0xFA,0x6B,0xEE,0xE4,0x61,0xA0,0x25,
  0x2F,0xAA,0x3B,0xBE,0xB4,0x31,0x13,0x96,0x9C,0x19,
  0x88,0x0D,0x07,0x82,0x86,0x03,0x09,0x8C,0x1D,0x98,
  0x92,0x17,0x35,0xB0,0xBA,0x3F,0xAE,0x2B,0x21,0xA4,
  0x65,0xE0,0xEA,0x6F,0xFE,0x7B,0x71,0xF4,0xD6,0x53,
  0x59,0xDC,0x4D,0xC8,0xC2,0x47,0xC5,0x40,0x4A,0xCF,
  0x5E,0xDB,0xD1,0x54,0x76,0xF3,0xF9,0x7C,0xED,0x68,
  0x62,0xE7,0x26,0xA3,0xA9,0x2C,0xBD,0x38,0x32,0xB7,
  0x95,0x10,0x1A,0x9F,0x0E,0x8B,0x81,0x04,0x89,0x0C,
  0x06,0x83,0x12,0x97,0x9D,0x18,0x3A,0xBF,0xB5,0x30,
  0xA1,0x24,0x2E,0xAB,0x6A,0xEF,0xE5,0x60,0xF1,0x74,
  0x7E,0xFB,0xD9,0x5C,0x56,0xD3,0x42,0xC7,0xCD,0x48,
  0xCA,0x4F,0x45,0xC0,0x51,0xD4,0xDE,0x5B,0x79,0xFC,
  0xF6,0x73,0xE2,0x67,0x6D,0xE8,0x29,0xAC,0xA6,0x23,
  0xB2,0x37,0x3D,0xB8,0x9A,0x1F,0x15,0x90,0x01,0x84,
  0x8E,0x0B,0x0F,0x8A,0x80,0x05,0x94,0x11,0x1B,0x9E,
  0xBC,0x39,0x33,0xB6,0x27,0xA2,0xA8,0x2D,0xEC,0x69,
  0x63,0xE6,0x77,0xF2,0xF8,0x7D,0x5F,0xDA,0xD0,0x55,
  0xC4,0x41,0x4B,0xCE,0x4C,0xC9,0xC3,0x46,0xD7,0x52,
  0x58,0xDD,0xFF,0x7A,0x70,0xF5,0x64,0xE1,0xEB,0x6E,
  0xAF,0x2A,0x20,0xA5,0x34,0xB1,0xBB,0x3E,0x1C,0x99,
  0x93,0x16,0x87,0x02,0x08,0x8D
};
#endif

int main(void) {
    double ax = 0;
    double ay = 0;
    double az = 0;

    #if DEBUG_ENABLED == 1
	char itmp[10];
	#endif

    //init uart
	uart_init(UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU));

	//init interrupt
	sei();

	//init mma7455
    adxl345_init();

    //do calibration
    //adxl345_calibrateoffset();

	for(;;) {
		adxl345_getdata(&ax, &ay, &az);

		#if DEBUG_ENABLED == 1
		dtostrf(ax, 3, 5, itmp); uart_puts(itmp); uart_putc(' ');
		dtostrf(ay, 3, 5, itmp); uart_puts(itmp); uart_putc(' ');
		dtostrf(az, 3, 5, itmp); uart_puts(itmp); uart_putc(' ');

		#if ADXL345_GETATTITUDE == 1
		double pitch = 0;
		double roll = 0;
		adxl345_getpitchroll(ax, ay, az, &pitch, &roll);
		pitch = pitch * 180 / M_PI; //radians to degree
		roll = roll * 180 / M_PI; //radians to degree
		dtostrf(pitch, 3, 5, itmp); uart_puts(itmp); uart_putc(' ');
		dtostrf(roll, 3, 5, itmp); uart_puts(itmp); uart_putc(' ');
		#endif

		uart_puts("\r\n");

		_delay_ms(500);
		#else
		long *ptr = 0;

		#if CRCCHECK_ENABLED == 1
	    unsigned char crcc = 0;
		crcc = 0;
		ptr = (long *)(&ax);
		crcc = pgm_read_word(&(crc8table[crcc ^ (unsigned char)*ptr]));
		crcc = pgm_read_word(&(crc8table[crcc ^ (unsigned char)(*ptr>>8)]));
		crcc = pgm_read_word(&(crc8table[crcc ^ (unsigned char)(*ptr>>16)]));
		crcc = pgm_read_word(&(crc8table[crcc ^ (unsigned char)(*ptr>>24)]));
		ptr = (long *)(&ay);
		crcc = pgm_read_word(&(crc8table[crcc ^ (unsigned char)*ptr]));
		crcc = pgm_read_word(&(crc8table[crcc ^ (unsigned char)(*ptr>>8)]));
		crcc = pgm_read_word(&(crc8table[crcc ^ (unsigned char)(*ptr>>16)]));
		crcc = pgm_read_word(&(crc8table[crcc ^ (unsigned char)(*ptr>>24)]));
		ptr = (long *)(&az);
		crcc = pgm_read_word(&(crc8table[crcc ^ (unsigned char)*ptr]));
		crcc = pgm_read_word(&(crc8table[crcc ^ (unsigned char)(*ptr>>8)]));
		crcc = pgm_read_word(&(crc8table[crcc ^ (unsigned char)(*ptr>>16)]));
		crcc = pgm_read_word(&(crc8table[crcc ^ (unsigned char)(*ptr>>24)]));
		#endif

		ptr = (long *)(&ax);
		uart_putc(*ptr);
		uart_putc(*ptr>>8);
		uart_putc(*ptr>>16);
		uart_putc(*ptr>>24);
		ptr = (long *)(&ay);
		uart_putc(*ptr);
		uart_putc(*ptr>>8);
		uart_putc(*ptr>>16);
		uart_putc(*ptr>>24);
		ptr = (long *)(&az);
		uart_putc(*ptr);
		uart_putc(*ptr>>8);
		uart_putc(*ptr>>16);
		uart_putc(*ptr>>24);

		#if CRCCHECK_ENABLED == 1
		uart_putc(crcc);
		#endif

		uart_putc('\n');
		#endif
	}

}
